var
	const
		ALL_INPUT_FIRST = 1
		ACT_IMMEDIATELY = 2
		ALTERNATE_SIDES = 3

battle_control
	var
		map_info/default_map
		list/battles = list()
		list/ids = list()
		// you could perhaps add variables to this datum, such as grass_map = 2.  Then send battle.map = init_map(grass_map).  It will make things easier to see later.

	proc
		// init_map is called when a new /battle is created.  It just copies the z level sent to it and returns the copied map.
		init_map(z)
			return maps.copy(z)

		// use this proc (call battle_control.new_battle()) to initiate a new battle.
		// send the attacker's party as the first argument
		// send the defender's party as the second argument
		// send a side_limit if you want one, otherwise it will be 0
		// and send a map_info.  do var/map_info/m = new /map_info/whatever_map and send m to the argument. See: pokemon-demo/turfs.dm
		new_battle(party/one, party/two, map_info/map = default_map, input_mode = ACT_IMMEDIATELY, side_limit = 0)
			if(!one || !two) return
			if(ispath(map))
				map = new map
			else map = new

			// set the parties' in_battle variable to TRUE
			one.in_battle = TRUE
			two.in_battle = TRUE

			// safety check on side_limit to prevent negatives
			if(side_limit < 0) side_limit = abs(side_limit)

			var
				list
					o = list()
					t = list()
					clients = list()
					all = list()
				client/c

			// add every member of both parties to the list/all
			// add every client of both parties to the list/clients
			// add every battle-capable mob in each party to list/o or list/t respectively
			// pop the current client's eye onto that battle-capable mob if the client isn't battle-capable
			for(var/mob/m in one.members)
				all.Add(m)
				if(m.client)
					c = m.client
					clients.Add(c)
				if(one.members[m])
					o.Add(m)
					if(c)
						if(!(c.mob in o))
							c.eye = m
						else
							c.eye = c.mob
			c = null
			for(var/mob/m in two.members)
				all.Add(m)
				if(m.client)
					c = m.client
					clients.Add(c)
				if(two.members[m])
					t.Add(m)
					if(c)
						if(!(c.mob in t))
							c.eye = m
						else
							c.eye = c.mob

			// generate a new tag for the battle (battles use the identifier "B")
			var/id = generate_tag("B", ids)
			src.ids.Add(id)
			//and initialize a new /battle and add it to the battles list
			src.battles += new /battle(id, o, t, map, input_mode, side_limit, clients, all)

		// this is called to delete a /battle.  Removes it from the battles list, removes it's tag from the ids list, and deletes it
		end_battle(battle/b)
			if(!(b in battles)) return
			battles.Remove(b)
			ids.Remove(b.tag)
			del b

battle
	var

		input_mode = ACT_IMMEDIATELY

		turn_control/turn_control
		side_limit = 0
		map_info/map
		Map/battle_map
		list/clients = list()

	// you shouldn't ever have to call New().  battle_control.new_battle() does that for you.
	// you'll end up having to do what new_battle() is already doing if you try to call it yourself.
	New(id, list/side_one, list/side_two, map_info/m, inp_mode = ACT_IMMEDIATELY, side_limit = 0, list/c, list/all)
		if(!m) del src
		if(!side_one.len || !side_two.len) del src

		src.side_limit = side_limit
		if(side_limit > 0)
			if(side_one.len > side_limit)
				side_one.Cut(side_limit + 1)
			if(side_two.len > side_limit)
				side_two.Cut(side_limit + 1)
		map = m
		tag = id
		input_mode = inp_mode
		if(c.len)
			clients = c.Copy()
		src.turn_control = new(src, side_one, side_two, all)

		// begin the battle
		begin()

	proc
		// this initializes the map, puts the mobs on it, and begins the turn_control
		begin()
			battle_map = battle_control.init_map(map.z)
			var/list/one = battle_map.get(/battle_info/side_one)
			var/list/two = battle_map.get(/battle_info/side_two)
			var/battle_info/dir_one
			var/battle_info/dir_two
			if(one.len)
				dir_one = one[1]
			if(two.len)
				dir_two = two[1]
			turn_control.locate_mobs(one, two, map.z, get_dir(dir_one, dir_two), get_dir(dir_two, dir_one))
			turn_control.begin()

		// this is called at the end of each turn to see if the battle is over.  you will HAVE to override this or your battle will never end.
		// you decide what determines is a battle is over.
		// You should return whichever side won (side_one or side_two), because it will be sent as an argument to end().
		// It doesn't do anything with it by default, but if you want to, the functionality is there, so long as you return which side won.
		is_over()
			. = null
			if(turn_control.side_one.run)
				. = turn_control.side_two
			if(turn_control.side_two.run)
				. = turn_control.side_one

		// this is called if is_over() returns a non-FALSE value.
		end(winner)
			reset_members()
			// and put the mobs back.  sending no arguments to locate_mobs() will put them back where they were (This won't work if you haven't sent them - it saves
			// 		their old locations when you do that)
			turn_control.locate_mobs()

			// end_override() is a proc that the user can override to do something before the battle ends.
			// if they want to do something before the mobs are relocated, they just need to redefine this proc and change the order of these 4 procs.
			end_override()

			// Now delete the /battle by calling battle_control.end_battle(src)
			battle_control.end_battle(src)

		reset_members()
			for(var/mob/m in turn_control.all_members)
				m.party.in_battle = FALSE

			for(var/client/c in clients)
				c.eye = c.mob

		end_override()